// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"
#include "kozu/resources.h"
#include "kozu/options.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying vec3 wpos;
varying vec3 kpos;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "util.h"
uniform float TIME;
uniform float RENDER_DISTANCE;
uniform vec2 FOG_CONTROL;
uniform vec4 FOG_COLOR;

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
	vec4 diff1 = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 diff1 = texture2D(TEXTURE_1, vec2(0,1));
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif
highp vec2 ttime = vec2(max(1.0-min(pow(diff1.r*1.8,5.0),1.0),0.0),pow(clamp((1.0-pow(diff1.r,5.0))*1.3,0.0,1.0),8.0))*uv1.y;
vec2 ftime = vec2(pow(max(min(1.0-FOG_COLOR.r*1.5,1.0),0.0),1.2),pow(max(min(1.0-FOG_COLOR.r*1.2,1.0),0.0),0.5));
#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif
    diffuse.rgb = saturation(diffuse.rgb,satamt);
#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

bool water = (color.b>color.r);
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);
bool Uwater = (FOG_CONTROL.x==0.0);
float cave = 1.0-uv1.y;
float rain = 1.0-pow(FOG_CONTROL.y, 11.0);
highp float hpTIME = TIME*1.33;
vec3 block = normalize(cross(dFdx(kpos),dFdy(kpos)));

///UNDERWATER
if(Uwater==true){
	highp float uwave = noise(cos(hpTIME+kpos.zxy+kpos.xzy+kpos.xyz+kpos.zxy+kpos.zyx));
        diffuse.rgb *= vec3(0.2,0.8,1.2);
		diffuse.rgb += uwave*0.6*(uv1.y*0.5);

}

///BLEND (WATER-FOG)
#if defined(BLEND)
diffuse.a = 0.43215644;
if(water==true){
	highp float wwave = noise(cos(hpTIME+kpos.zxy+kpos.xzy+kpos.xyz+kpos.zxy+kpos.zyx));
        diffuse.rgb += wwave*0.08;
        diffuse.rgb = saturation(vec3(0.2,0.8,1.2), satamt);
		diffuse.rgb += noise(wpos*0.5*abs(wwave))*0.5;
}
#endif

float DL = max(0.0, max(block.z, -block.z));
diffuse.rgb *= mix(1.0,0.6,DL); 
diffuse.rgb = TMap(diffuse.rgb,ttime);
diffuse.rgb *= mix(vec3(1.0,1.0,1.0),vec3(0.6,0.6,0.6),smoothstep(0.58,0.47,color.g));
diffuse.rgb *= mix(mix(0.58,1.0,smoothstep(0.850,0.870,uv1.y)),1.0,uv1.x);
    if(nether==true){
        diffuse.rgb *= diffuse.rgb*1.08;
    }
	
///FOG
vec3 fogg = mix(diffuse.rgb,FOG_COLOR.rgb,clamp(length(-wpos*0.5)/RENDER_DISTANCE*2.0,0.0,0.7));
diffuse.rgb = fogg;

#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}
