#include "ShaderConstants.fxh"
#include "util.fxh"

#include "kozu/resources.hlsl"
#include "kozu/options.hlsl"

struct PS_Input
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
	float3 wpos : WPOSS;
	float3 kpos : KPOG;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

ROOT_SIGNATURE

float3 exp(float3 diffuse){
     float gamma = 0.0;
     float exposure = 1.7;
     diffuse *= exposure/(1.0 + diffuse / exposure);
return diffuse;
}


void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
	float4 diffl = texture2D_AA(TEXTURE_1, TextureSampler1, float2(0.0, 1.0));
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
	float4 diffl = TEXTURE_1.Sample(TextureSampler1, float2(0.0, 1.0));
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif
 
#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif


bool water = (PSInput.color.b>PSInput.color.r);

bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);

bool Uwater = (FOG_CONTROL.x==0.0);

float cave = 1.0-PSInput.uv1.y;

float rain = 1.0-pow(FOG_CONTROL.y, 11.0);

float circle = clamp(1.0-length(PSInput.wpos.zx+float2(.0,5.))/RENDER_DISTANCE*50.0,0.0,0.8);

float3 waterc = lerp(float3(0.1,0.3,0.8),float3(1.8,1.8,1.8),clamp(length(PSInput.wpos)/RENDER_DISTANCE*1.0,0.0,2.0));

float wwave = noise(sin(TIME*W_WAVE_SPEED+PSInput.kpos.zxy+PSInput.kpos.xzy+PSInput.kpos.xyz+PSInput.kpos.zxy+PSInput.kpos.zyx));

float3 block = normalize(cross(ddx(PSInput.kpos),ddy(PSInput.kpos)));

float lline = smoothstep(1.5, .0, abs(PSInput.wpos.z+float2(3.0,0.0)));

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif
float2 dt = 
    float2(max(1.0-min(pow(diffl.r*1.8,5.0),1.0),0.0),
	pow(clamp((1.0-pow(diffl.r,5.0))*1.3,0.0,1.0),
	8.0))*PSInput.uv1.y;
#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif
    diffuse.rgb = saturation(diffuse.rgb,satamt);
#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

///UNDERWATER
if(Uwater==true){
        diffuse.rgb *= float3(0.4,1.0,1.4);
		diffuse.rgb += wwave*0.6*(PSInput.uv1.y*0.5);
    }

/// BLEND / TRANSLUCENT
#ifdef BLEND
    if(water==false){
	    float sunReflection = circle*float4(diffuse.r*diffuse.r,diffuse.g*diffuse.g,diffuse.b*diffuse.b,0.95);
        diffuse.rgb += sunReflection;
	}
		if(water==true){
	        diffuse.rgb *= waterc;
	        diffuse.a = 0.43215644;
			float sunReflection = circle*float4(diffuse.r+diffuse.r+diffuse.r,diffuse.g+diffuse.g+diffuse.g,diffuse.b+diffuse.b+diffuse.b,0.99);
            diffuse.rgb += sunReflection;
            diffuse.rgb = saturation(diffuse.rgb, 1.0);
		    diffuse.rgb += wwave*0.09;
	    }

#endif

///COLOUR AND SHADING
float DLZ = max(0.0, max(block.z, -block.z));
diffuse.rgb *= lerp(1.0,0.7,DLZ); 
diffuse.rgb = TMap(diffuse.rgb,dt,rain);
diffuse.rgb *= lerp(lerp(CLR_SHD,1.0,smoothstep(0.76,0.85,PSInput.uv1.y)),1.0,PSInput.uv1.x);
diffuse.rgb *= lerp(float3(1.0,1.0,1.0),float3(0.75,0.75,0.75),smoothstep(0.56,0.46,PSInput.color.g));
    if(nether==true){
        diffuse.rgb *= diffuse.rgb*1.08;
    }

///FOG
float3 fogg = lerp(diffuse.rgb,FOG_COLOR.rgb,clamp(length(-PSInput.wpos.xz*0.5)/RENDER_DISTANCE*2.0,0.0,0.4));
diffuse.rgb = fogg; 

#ifdef FOG
    diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a ); 
#endif
	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}